function Artefact_Correction_input(dataset)

% the following flags should all be false for reproducing the paper, and, 
% unless you really know what you are doing, for your own data,
% but can be changed for playing around

minimal = false; % algorithm uses only minimal input, time axis, two traces, number of shifted points
autotime = false; % automatic determination of artefact zero time overrides external input
second_fit = false; % secondary amplitude fit of artefact by balancing corrected traces, only minimally tested, not recommended 

compare_4DEER = true; % 4-pulse DEER data is fitted to corrected 5-pulse DEER trace 

if ~exist('dataset','var'),
    dataset = 1;
end;

switch dataset
    case 1
        whichplot = 'WALP_2_22';
    case 2
        whichplot = 'WALP_7_22';
    case 3
        whichplot = 'WALP_2_22_short';
    case 4
        whichplot = 'Nanodisc_2d_8us';
    case 5
        whichplot = 'Nanodisc_2d_10us';
    case 6
        whichplot = 'WALP_7_22_rectangular_Q';
    case 7
        whichplot = 'Sim_input_Bimodal';
    otherwise
        fprintf(2,'Warning. Only 7 example spectra are provided. Defaulting to spectrum 1\n');
        whichplot = 'WALP_2_22';
end;

% add all experimental data subfolders
addpath(genpath(pwd));

% choose if manual zerotime
Manual_zerotime_Artefact_flag = true;
Manual_zerotime_5_flag = true; 
z_4 = -29;      % Shift of zerotime for part. exc. artefact in ns. Determine beforehand for specific pulse settings on strong sample with short tau. Here: HS16, 100ns, 150 MHz wide.
z_5 = -29;      % Shift of zerotime for main signal in ns. Determine beforehand for specific pulse settings on strong sample with short tau. Here: HS16, 100ns, 150 MHz wide.

cutdata = 1; % cutdata is which percentage of the input data (after zero time of main signal) should be used, i.e. 0.8*the length of the trace
data_2D_flag = true;
Manual_ZeroTime_4PulseDEER_flag = false;

clear input
clear fileslist
switch whichplot
    case 'WALP_2_22'           
        % 5-pulse DEER data
        fileslist{1} = '2d_5pDEER_WALP2_22_HS16_100ns_150MHz_rlcmod_4200d2_171sc';    
        tau=4200; 
        t0=200; 
        l=5; 
        shift_vec=l*8;   
        tprime_start(1) = 80; % initial delay before moving pump pulse, d3 in PulseSpel
        
        % 4-pulse DEER data
        deer4ref.name='deer_WALP23_2_22_50K_34410_34310_3500_500sc_211113';
        input.bg_mode='fractal';
        
    case 'WALP_7_22'          
        % 5-pulse DEER data
        fileslist{1} = '2D_5pDEER_HS16_rlcmod_NuclMod_8x16nsIncrement_34sc'; 
        Manual_zerotime_Artefact_flag = false;
        tau=3000; 
        t0=200; 
        tprime_start(1) = 100; % in pulse spel d3       
        l = 5;         
        shift_vec=l*8;  % difference in artefact position
        
        % 4-pulse DEER data
        deer4ref.name='deer4_WALP23DOPC_batch1p5_s1_50K_34640_34540_2700_80sc_051213';
        input.bg_mode='fractal';
        
    case 'WALP_2_22_short'         
        % 5-pulse DEER data
        fileslist{1} = '2d_5pDEER_WALP2_22_HS16_100ns_150MHz_rlcmod_2200d2_38sc';  
        tau=2200; 
        t0=200;  
        l=5;        
        shift_vec=l*8;  
        tprime_start(1) = 80; % initial delay before moving pump pulse, d3 in PulseSpel
        
        % 4-pulse DEER data
        deer4ref.name='deer_WALP23_2_22_50K_34410_34310_3500_500sc_211113';
        input.bg_mode='fractal';
        
    case 'Nanodisc_2d_8us';                  
        % 5-pulse DEER data
        tau=8000; 
        t0=200;  
        fileslist{1} = '2d_5pDEER_ND192_NuclMod_HS16_100ns_50MHzOffset_150MHzwide_72sc'; 
        l=12;         
        shift_vec=l*16;  
        tprime_start(1) = 120;  
        
        % 4-pulse DEER data
        deer4ref.name='deer_ND192Bib50uM_10dGly250315_50K_34520_34420__5000_325sc_270315_bg3fix_st1600st_reg100fit';
        input.bg_mode='fractal';
        
    case 'Nanodisc_2d_10us';          
         % 5-pulse DEER data
         tau=10000;
         t0=200;   
         tprime_start(1) = 100; 
         l=12; 
         cutdata = 0.995; %0.995; 
         fileslist{1} = '2d_5pDEER_ND192_HS16_100ns_70MHzOffs_150MHz_rlc_56sc'; 
         shift_vec(1)=l*16;  

         % 4-pulse DEER data
         deer4ref.name='deer_ND192Bib50uM_10dGly250315_50K_34520_34420__5000_325sc_270315_bg3fix_st1600st_reg100fit';
         input.bg_mode='fractal';
         
     case 'WALP_7_22_rectangular_Q'   
        % 5-pulse DEER data
        z_4 = +25;     z_5 = +56; % shifts in ns for rectangular pulses at Q band (not determined experimentally but from sharp peaks in WALP7-22)
        fileslist{1} = '2D_5pDEER_WALP7_22_rect13ns_rescmp_2000d2_38sc';  
        tau=2000; 
        t0=300;   
        cutdata = 0.999; 
        tprime_start(1) = 80; 
        l=5;
        shift_vec=l*8;    
        
        % 4-pulse DEER data
        deer4ref.name='deer4_WALP23DOPC_batch1p5_s1_50K_34640_34540_2700_80sc_051213';
        input.bg_mode='fractal';

     case 'Sim_input_Bimodal'           
        % 5-pulse DEER data
        fileslist{1} = 'Sim_rmeanc_32_sigr_3_A4p_20_noise_20_tau_9000_t0_200';    
        fileslist{2} = 'Sim_rmeanc_32_sigr_3_A4p_20_noise_20_tau_9000_t0_392';    
        data_2D_flag = false; 
        n_vec(1) = 1; n_vec(2) = 1;  
        Manual_zerotime_Artefact_flag = true;
        Manual_zerotime_5_flag = false; 
        cutdata = 0.97; 
        tau=9000; 
        t0=200; 
        l=192/8;       
        shift_vec=l*8;   
        tprime_start(1) = 0; % initial delay before moving pump pulse, d3 in PulseSpel
                
        % 4-pulse DEER data
        deer4ref.name='Sim_rmeanc_32_sigr_3_A4p_1_noise_20_tau_9000_t0_200_Lalpha';
        input.bg_mode='none';
end

tshift=tau/2-t0;  

if Manual_zerotime_5_flag;
    Manual_zerotime_5 = t0+z_5-tprime_start(1)    ; % in ns
end
if Manual_zerotime_Artefact_flag;
    if Manual_zerotime_5_flag;
        Manual_zerotime_Artefact = tau/2+z_4-(t0+z_5)  ; 
    else
        Manual_zerotime_Artefact = tshift; 
    end
end

clear t1, clear v1, 
if data_2D_flag
    currfile = mat2str(fileslist{1})
    [t1,~,vp,vb1] = get_elexsys(currfile(2:(end-1))); 
        t2=t1;     
        v1=vp(1,:);
        v2=vp(2,:);
        nscan1 = 1; % 2d data has the same number of scans for both traces
        nscan2 = nscan1;
        ds = round(shift_vec/((t2(2)-t2(1)))); 
else
            currfile1 = mat2str(fileslist{1}); 
            [t1,~,v1,vb1] = get_elexsys(currfile1(2:(end-1))); 
            currfile2 = mat2str(fileslist{2}); 
            [t2,~,v2,vb2] = get_elexsys(currfile2(2:(end-1))); 
        switch whichplot
            case 'Sim_input_Bimodal'
                t1 = t1 *1000; 
                t2 = t2 *1000; 
        end
        nscan1 = n_vec(1);
        nscan2 = n_vec(2);
        ds = round(shift_vec/((t2(2)-t2(1)))); 
end

% Phase correction
phi1 = get_phase(v1);
phi2 = get_phase(v2);
v1 = real(exp(1i*phi1)*v1)/nscan1;
v2 = real(exp(1i*phi2)*v2)/nscan2;

dt1=t1(2)-t1(1);
dt2=t2(2)-t2(1); 
if dt2 ~= dt1,
    error('Time increments of the two traces do not match.');
end;

% Find zerotime
[zp1,t01] = get_zt(t1,v1);
    t1z = t1(zp1:end) - t1(zp1);
    v1z = v1(:,zp1:end);
[zp2,t02] = get_zt(t2,v2);
    t2z = t2(zp2:end) - t2(zp2);
    v2z = v2(:,zp2:end);

if Manual_zerotime_5_flag; 
    zp1_man = round(Manual_zerotime_5/dt1); 
    zp2_man = round(Manual_zerotime_5/dt1); 
    if zp1_man == 0; zp1_man = 1; end 
    if zp2_man == 0; zp2_man = 1; end 
    t1z = t1(zp1_man:end) - t1(zp1_man);
    v1z = v1(:,zp1_man:end);
    t2z = t2(zp2_man:end) - t2(zp2_man);
    v2z = v2(:,zp2_man:end);
end

figure(778); clf;  hold on
plot(t1z(1:round(cutdata*length(v1z)))/1000,v1z(1:round(cutdata*length(v1z))),'LineWidth',3,'Color',[0 114 189]./250);
plot(t2z(1:round(cutdata*length(v2z)))/1000,v2z(1:round(cutdata*length(v2z))),'LineWidth',3,'Color',[126 47 142]./250);
  
input.trace1 = v1z;
input.t1 = t1z;
input.trace2 = v2z;
input.t2 = t2z;

if compare_4DEER;
    loadname=[deer4ref.name '_bckg.dat']; % according to the format of DEERAnalysis
    dat4deer=load(loadname);
    t4=dat4deer(:,1); % time axis in microsec
    tr4=dat4deer(:,2); % bg-uncorrected trace
    bg4deer=dat4deer(:,3); % bg fit;

    % adjust 4DEER trace according to the 5DEER traces
    dt4=t4(2)-t4(1);
    [~,in]=min(abs(t4-0));
    if Manual_ZeroTime_4PulseDEER_flag;
        in = Manual_ZeroTime_4PulseDEER; 
    end
    t4in=t4(in:end);
    tr4in=tr4(in:end)-bg4deer(in:end);
    tr4in = tr4in/max(tr4in); 
    t4ins=t4in(1):dt1/1000:max(t4in);  
    tr4ins=interp1(t4in,tr4in,t4ins,'spline');
end

% input parameters for artefact correction
input.degree=3;
if ~minimal,
    input.tshift=tshift;        % relative timing of the partial excitation artefact after the ideal 5-pulse signal
end;
input.ds=ds;                % difference in artefact position in the two traces. Used for search window.
input.cutdata= cutdata; 	
input.interp = false;  % Whether interpolation of time axis to 0.5 ns wanted
if ~minimal && ~autotime,
    if Manual_zerotime_Artefact_flag; 
        input.ext_zt = Manual_zerotime_Artefact; % Zero-time of artefact after main signal. CAREFUL, not yet tested with .interp being true.
        input.corr_shift = true;    % manual time shift of artefact timing
    end
end;
input.sec_fit = second_fit;   % zero time shift and amplitude correction by second fit
input.saveASCII=1;

% input parameters for fitting of 4-DEER background onto 5-DEER
if compare_4DEER;
    input.compare4DEER_flag = true;
    input.exp_depth=0.53;
    input.fit_depth=1;
    input.tr4=tr4ins/max(tr4ins);
    input.t4=t4ins*1000;
else 
    input.compare4DEER_flag = false;   
end

% artefact correction
[t,deer] = artefact_correction(input);

% plot result
figure(777);
plot(t,deer,'k');
title('Corrected DEER trace');
set(gca,'FontSize',14);

figure(778); hold on
plot(t/1000,deer,'LineWidth',2,'Color',[31 64 122]./250);
legend('t_0 1','t_0 2','corrected trace')
h=figure(778); h.CurrentAxes.FontSize = 14; 
ylabel('Intensity (a.u.)'); xlabel('\itt \rm(\mus)')
figure(778); hold on

savename = [fileslist{1} '_Corrected.dat'];
Corrected_t(:,1) = t;
Corrected_t(:,2) = deer;
save(savename,'Corrected_t','-ascii');
fprintf(1,'--- Correction finished ---\nData saved to file: %s\n',savename);

function phi = get_phase(data)
% Determines phase phi that minimizes imaginary part of a data trace
%
% G. Jeschke, 22.10.2016

phi=fminbnd(@rms_phi_2D,-pi/2+1e-4,pi/2,[],data);

function rms=rms_phi_2D(phi,data)
% r.m.s. of imaginary part after phase correction
itr=imag(data*exp(1i*phi));
rms=sqrt(sum(sum(itr.*itr)));

function [zp,t0] = get_zt(t,data)

cdata = sum(real(data),1);
% Determine maximum
[~,mp]=max(cdata);
nmp=1;
% Determine time zero by moment analysis
if mp > 1 && mp < length(cdata),
    dmi = mp-1;
    dma = length(cdata)-mp;
    dm = dmi; if dma < dm, dm = dma; end;
    maxd = floor(dm/2);
    dev = 1e20;
    nmp = mp;
    for k = -maxd+mp:maxd+mp,
        summ = 0;
        for l = -maxd:maxd,
            summ = summ+cdata(k+l)*l;
        end;
        if abs(summ)<dev, dev = abs(summ); nmp=k; end;
    end;
end;
t0 = t(nmp);
zp = nmp;



function [hp,th] = get_half_decay(t,data,options)
% Determines time, where the modulation in the signal has decayed to half
% the initial value

% expoential background fit of normalized data
tr = data/max(data);
ps = round(length(t)/4);
poly = polyfit(t(ps:end),log(tr(ps:end)),1); % linear fit of logarithm
bckg = exp(polyval(poly,t)); % background is exponential of that
mod_depth = 1-bckg(1);
hd = 1 - mod_depth/2;
[~,hp] = min(abs(tr-hd));
th = t(hp);
if exist('options','var') && options.verbose,
    figure;
    plot(t,tr,'k');
    hold on;
    plot(t,bckg,'r');
    plot([th,th],[0,1],'b:');
    fprintf(1,'Time of half decay is : %4.1f ns\n',th);
end;

