function rmsd = rmsd_corr_diff(v,ffan,ffbn,rmsd0,corrected,dt,t1,ds)

persistent min_rmsd best_fit % the function has a memory for the best 
                             % achieved rmsd and the best previous
                             % corrected signal

if isempty(min_rmsd), % initialize
    min_rmsd = rmsd0;
end;

if isempty(best_fit),
    best_fit = corrected;
end;
    
corrected = best_fit;
sc = v(1);
t0 = v(2);
ni = ceil(log(1e-3)/log(sc)); % dampen error by 10^-3
ti = -(length(corrected)-1)*dt:dt:(length(corrected)-1)*dt; % time axis for iterated data
for ki = 1:ni, 
    iterate = [fliplr(corrected(2:end)) corrected]; % symmetric version
    iterate1 = sc*interp1(ti+t0,iterate,t1); % new estimate
    iterate2 = sc*interp1(ti+t0-ds*dt,iterate,t1); % time shift and scaled artefact estimate
    corra = ffan - iterate1; % new corrected trace 1 with current artefact estimate
    corrb = ffbn - iterate2; % new corrected trace 2 with current artefact estimate
    corrected = (corra+corrb)/2; % mean corrected trace
end;

rmsd = sqrt(sum((corra-corrb).^2)/length(corra));

if rmsd < min_rmsd, % store rmsd and corrected signal, if it is the best so far
    min_rmsd = rmsd;
    best_fit = corrected;
end;